/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef ENCRYPTER_H
#define ENCRYPTER_H

#include "core/outboundpkt.h"
#include "types/decryptedmessage.h"
#include "secret/secretchat.h"

class Encrypter : public OutboundPkt
{
public:
    explicit Encrypter();

    void setSecretChat(SecretChat *secretChat);

    QByteArray generateEncryptedDataForSendMessage(qint64 randomId, const QString &message);
    QByteArray generateEncryptedDataForNotifyLayer(qint64 randomId, qint32 layer);
    QByteArray generateEncryptedDataForSendPhoto(qint64 randomId, const QByteArray &key, const QByteArray &iv,
                                                 const QByteArray &thumb = QByteArray(), qint32 thumbW = 0, qint32 thumbH = 0,
                                                 qint32 size = 0, qint32 width = 0, qint32 height = 0);
    QByteArray generateEncryptedDataForSendVideo(qint64 randomId, const QByteArray &key, const QByteArray &iv,
                                                 const QByteArray &thumb = QByteArray(), qint32 thumbW = 0, qint32 thumbH = 0,
                                                 const QString &mimeType = QString(),
                                                 qint32 duration = 0, qint32 size = 0, qint32 width = 0, qint32 height = 0);


protected:
    qint32 *mEncrExtra;
    qint32 *mEncrPtr;
    qint32 *mEncrStart;
    qint32 *mEncrEnd;
    SecretChat *mSecretChat;

    void startEncryption();
    void endEncryption();
    qint32 *encryptDecryptedMessage();
    QByteArray getGeneratedBytes();

    QByteArray generateRandomBytes();
    QByteArray generateEncryptedData(const DecryptedMessage &decryptedMessage);

    void appendDecryptedMessage(const DecryptedMessage &decryptedMessage);
    void appendDecryptedMessageMedia(const DecryptedMessageMedia &media);
    void appendDecryptedMessageAction(const DecryptedMessageAction &action);
    void appendSendMessageAction(const SendMessageAction &sendMessageAction);
};

#endif // ENCRYPTER_H
